Ptr

Struct Ptr 

pub struct Ptr<T> { /* private fields */ }
A transparent wrapper around Option<NonNull<T>> returned by AtomicTaggedPtr operations.

It provides convenient helper methods to convert into raw const/mutable pointers, access the underlying Option<NonNull<T>>, and supports direct comparisons.

Implementations

Source

impl<T> Ptr<T>

pub const fn new(ptr: Option<NonNull<T>>) -> Self

Creates a new Ptr wrapper from an Option<NonNull<T>>.

pub fn as_mut_ptr(self) -> *mut T

Converts the pointer into a raw mutable pointer *mut T. Returns a null pointer if the underlying value is None.

pub fn as_ptr(self) -> *const T

Converts the pointer into a raw const pointer *const T. Returns a null pointer if the underlying value is None.

pub const fn option(self) -> Option<NonNull<T>>

Obtains the underlying Option<NonNull<T>>.

pub const fn as_option(self) -> Option<NonNull<T>>

Obtains the underlying Option<NonNull<T>>.

pub fn is_null(self) -> bool

Returns true if the pointer is null.

pub fn is_some(self) -> bool

Returns true if the pointer is not null (is some).

pub fn is_none(self) -> bool

Returns true if the pointer is null (is none).

pub unsafe fn as_ref<'a>(self) -> Option<&'a T>

Returns a shared reference to the value if the pointer is not null.

Safety

The caller must ensure that the pointer is valid, aligned, points to a valid initialized value, and respects the aliasing rules of Rust references.

pub unsafe fn as_mut<'a>(self) -> Option<&'a mut T>

Returns a mutable reference to the value if the pointer is not null.

Safety

The caller must ensure that the pointer is valid, aligned, points to a valid initialized value, and respects the aliasing rules of Rust references.

pub fn expect(self, msg: &str) -> NonNull<T>

Unwraps the inner NonNull<T>, panicking with the given message if it is None.

pub fn unwrap(self) -> NonNull<T>

Unwraps the inner NonNull<T>, panicking if it is None.

pub fn unwrap_or(self, default: NonNull<T>) -> NonNull<T>

Returns the contained NonNull<T> or a default.

pub fn map<U, F>(self, f: F) -> Ptr<U>
where F: FnOnce(NonNull<T>) -> NonNull<U>,

Maps the inner NonNull<T> pointer to a new pointer of another type.

pub fn map_or<U, F>(self, default: U, f: F) -> U
where F: FnOnce(NonNull<T>) -> U,

Maps the inner NonNull<T> pointer to a value, or returns a default value.

pub fn map_or_else<U, D, F>(self, default: D, f: F) -> U
where D: FnOnce() -> U, F: FnOnce(NonNull<T>) -> U,

Maps the inner NonNull<T> pointer to a value, or evaluates a default closure.