TaggedPtr

Struct TaggedPtr 

pub struct TaggedPtr<T> {
    pub ptr: Ptr<T>,
    pub tag: Tag,
}
A packaged representation of a pointer and a generation tag. Used for atomic operations with AtomicTaggedPtr.

Fields

ptr: Ptr<T>

The physical pointer wrapper.

tag: Tag

The generation tag for ABA protection.

Implementations

Source

impl<T> TaggedPtr<T>

pub fn new<P>(ptr: P, tag: Tag) -> Self
where P: Into<Ptr<T>>,

Creates a new TaggedPtr from a pointer and a tag.

pub fn decompose(self) -> (Ptr<T>, Tag)

Deconstructs the TaggedPtr into a tuple of (Ptr<T>, Tag).

pub fn as_ptr(self) -> *const T

Converts the pointer into a raw const pointer *const T. Returns a null pointer if the underlying value is None.

pub fn as_mut_ptr(self) -> *mut T

Converts the pointer into a raw mutable pointer *mut T. Returns a null pointer if the underlying value is None.

pub fn is_null(self) -> bool

Returns true if the pointer is null.

pub fn is_some(self) -> bool

Returns true if the pointer is not null (is some).

pub fn is_none(self) -> bool

Returns true if the pointer is null (is none).

pub unsafe fn as_ref<'a>(self) -> Option<&'a T>

Returns a shared reference to the value if the pointer is not null.

Safety

The caller must ensure that the pointer is valid, aligned, points to a valid initialized value, and respects the aliasing rules of Rust references.

pub unsafe fn as_mut<'a>(self) -> Option<&'a mut T>

Returns a mutable reference to the value if the pointer is not null.

Safety

The caller must ensure that the pointer is valid, aligned, points to a valid initialized value, and respects the aliasing rules of Rust references.

pub fn with_ptr<U>(self, ptr: impl Into<Ptr<U>>) -> TaggedPtr<U>

Returns a new TaggedPtr with a different pointer but the same tag.

pub fn with_tag(self, tag: Tag) -> Self

Returns a new TaggedPtr with a different tag but the same pointer.

pub fn map_ptr<U, F>(self, f: F) -> TaggedPtr<U>
where F: FnOnce(Ptr<T>) -> Ptr<U>,

Maps the pointer part of the TaggedPtr using the given closure.